@@ -47,6 +47,11 @@ try:
 except ImportError:
     log.error("dbus is required for PC send fax.")
 
+import warnings
+# Ignore: .../dbus/connection.py:242: DeprecationWarning: object.__init__() takes no parameters
+# (occurring on Python 2.6/dBus 0.83/Ubuntu 9.04)
+warnings.simplefilter("ignore", DeprecationWarning)
+
 
 # Update queue values (Send thread ==> UI)
 STATUS_IDLE = 0
@@ -250,6 +255,7 @@ class FaxAddressBook(object): # Pickle based address book
                 self._data[new_name]
             except KeyError:
                 self._data[new_name] = self._data[old_name].copy()
+                self._data[new_name]['name'] = new_name
                 del self._data[old_name]
                 self.save()
 
@@ -329,7 +335,12 @@ class FaxAddressBook(object): # Pickle based address book
 
     def add_to_group(self, group, members):
         group_members = self.group_members(group)
-        self.update_groups(group, group_members + members)
+        new_group_members = []
+        for m in members:
+            if m not in group_members:
+                new_group_members.append(m)
+
+        self.update_groups(group, group_members + new_group_members)
 
 
     def remove_from_group(self, group, remove_members):
@@ -832,6 +843,7 @@ class FaxSendThread(threading.Thread):
                          "application/x-perl",
                          "application/x-python",
                          "application/x-shell",
+                         "application/x-sh",
                          "text/plain",]:
 
             cups.addOption('prettyprint')
