@@ -144,7 +144,7 @@ int __attribute__ ((visibility ("hidden"))) generalize_serial(const char *sz, ch
       buf[j++] = pMd[i];
    }
 
-   for (i--; buf[i] == '_' && i > 0; i--);  /* eat trailing white space */
+   for (i--; buf[i] == ' ' && i > 0; i--);  /* eat trailing white space */
 
    buf[++i] = 0;
 
@@ -191,6 +191,14 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) service_to_channel(mud
    {
       *index = HPMUD_SOAPSCAN_CHANNEL;
    }
+   else if (strncasecmp(sn, "hp-soap-fax", 11) == 0)
+   {
+      *index = HPMUD_SOAPFAX_CHANNEL;
+   }
+   else if (strncasecmp(sn, "hp-marvell-scan", 15) == 0)
+   {
+      *index = HPMUD_MARVELL_SCAN_CHANNEL;
+   }
    /* All the following services require MLC/1284.4. */
    else if (pd->io_mode == HPMUD_RAW_MODE || pd->io_mode == HPMUD_UNI_MODE)
    {
@@ -226,6 +234,10 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) service_to_channel(mud
    {
       *index = HPMUD_DEVMGMT_CHANNEL;
    }
+   else if (strncasecmp(sn, "hp-wificonfig", 13) == 0)
+   {
+      *index = HPMUD_WIFI_CHANNEL;
+   }
    else
    {
       BUG("invalid service=%s %s\n", sn, pd->uri);
@@ -411,7 +423,9 @@ int hpmud_get_uri_model(const char *uri, char *buf, int buf_size)
 int hpmud_get_uri_datalink(const char *uri, char *buf, int buf_size)
 {
    char *p;
+   char ip[HPMUD_LINE_SIZE];
    int i;
+   int zc=0;
 
    buf[0] = 0;
 
@@ -419,11 +433,29 @@ int hpmud_get_uri_datalink(const char *uri, char *buf, int buf_size)
       p+=7;
    else if ((p = strcasestr(uri, "ip=")) != NULL)
       p+=3;
+   else if ((p = strcasestr(uri, "zc=")) != NULL)
+   {
+      p+=3;
+      zc=1;
+   }
    else
       return 0;
 
-   for (i=0; (p[i] != 0) && (p[i] != '&') && (i < buf_size); i++)
-      buf[i] = p[i];
+   if (zc)
+   {
+#ifdef HAVE_LIBNETSNMP
+      if (hpmud_mdns_lookup(p, HPMUD_MDNS_TIMEOUT, ip) != HPMUD_R_OK)
+	 return 0;
+      for (i=0; (ip[i] != 0) && (i < buf_size); i++)
+         buf[i] = ip[i];
+#else
+      return 0;
+#endif
+   }
+   else {
+      for (i=0; (p[i] != 0) && (p[i] != '&') && (i < buf_size); i++)
+         buf[i] = p[i];
+   }
 
    buf[i] = 0;
 
