@@ -87,21 +87,22 @@ static char *fd_name[MAX_FD] =
    "ff/3/1",
    "ff/ff/ff",
    "ff/d4/0",
+   "ff/cc/0",
 };
 
 static int fd_class[MAX_FD] =
 {
-   0,0x7,0x7,0xff,0xff,0xff,0xff,0xff,
+   0,0x7,0x7,0xff,0xff,0xff,0xff,0xff,0xff,
 };
 
 static int fd_subclass[MAX_FD] =
 {
-   0,0x1,0x1,0x1,0x2,0x3,0xff,0xd4,
+   0,0x1,0x1,0x1,0x2,0x3,0xff,0xd4,0xcc,
 };
 
 static int fd_protocol[MAX_FD] =
 {
-   0,0x2,0x3,0x1,0x1,0x1,0xff,0,
+   0,0x2,0x3,0x1,0x1,0x1,0xff,0,0,
 };
 
 static const unsigned char venice_power_on[] = {0x1b, '%','P','u','i','f','p','.','p','o','w','e','r',' ','1',';',
@@ -563,20 +564,19 @@ static int release_interface(file_descriptor *pfd)
 /* Claim any open interface which is valid for device_id and device status. */
 static int claim_id_interface(struct usb_device *dev)
 {
-   int fd[] = {FD_7_1_2, FD_7_1_3, FD_ff_ff_ff, FD_ff_d4_0, FD_ff_1_1, FD_ff_2_1, FD_NA};
-   int i;
+   enum FD_ID i;
 
-   for (i=0; fd[i]!=FD_NA; i++)
+   for (i=FD_7_1_2; i!=MAX_FD; i++)
    {
-      if (get_interface(dev, fd[i], &fd_table[fd[i]]) == 0)
+      if (get_interface(dev, i, &fd_table[i]) == 0)
       {
-         if (claim_interface(libusb_device, &fd_table[fd[i]]))
+         if (claim_interface(libusb_device, &fd_table[i]))
             continue;  /* interface is busy, try next interface */
          break;  /* done */
       }
    }
 
-   return fd[i];
+   return i;
 }
 
 /* See if this usb device and URI match. */
@@ -634,6 +634,52 @@ bugout:
    return stat;
 }
 
+/* See if this usb device and serial number match. Return model if match. */
+static int is_serial(struct usb_device *dev, const char *sn, char *model, int model_size)
+{
+   usb_dev_handle *hd=NULL;
+   char sz[128];
+   char gen[128];
+   int r, stat=0;
+
+   if ((hd = usb_open(dev)) == NULL)
+   {
+      BUG("invalid usb_open: %m\n");
+      goto bugout;
+   }
+
+   if (dev->descriptor.idVendor != 0x3f0)
+      goto bugout;      /* not a HP product */
+
+   if ((r=get_string_descriptor(hd, dev->descriptor.iSerialNumber, sz, sizeof(sz))) < 0)
+   {
+      BUG("invalid serial id string ret=%d\n", r);
+      goto bugout;
+   }
+   if (sz[0])
+      generalize_serial(sz, gen, sizeof(gen));
+   else
+      strcpy(gen, "0");
+
+   if (strncmp(sn, gen, sizeof(gen)) != 0)
+      goto bugout;  /* match failed */
+
+   if ((r=get_string_descriptor(hd, dev->descriptor.iProduct, sz, sizeof(sz))) < 0)
+   {
+      BUG("invalid product id string ret=%d\n", r);
+      goto bugout;
+   }
+   generalize_model(sz, model, model_size);
+
+   stat = 1;    /* found usb device that matches sn */
+     
+bugout:
+   if (hd != NULL)
+      usb_close(hd);
+
+   return stat;
+}
+
 static struct usb_device *get_libusb_device(const char *uri)
 {
    struct usb_bus *bus;
@@ -1065,7 +1111,7 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_open(mud_device *
    {
       /* First client. */
 
-      if ((fd = claim_id_interface(libusb_device)) == FD_NA)
+      if ((fd = claim_id_interface(libusb_device)) == MAX_FD)
       {
          stat = HPMUD_R_DEVICE_BUSY;
          goto blackout;
@@ -1140,7 +1186,7 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_get_device_id(mud
       if (fd == FD_NA)
       {
          /* Device not in use. Claim interface, but release for other processes. */
-         if ((fd = claim_id_interface(libusb_device)) != FD_NA)
+         if ((fd = claim_id_interface(libusb_device)) != MAX_FD)
          {
             *len = device_id(fd, pd->id, sizeof(pd->id));  /* get new copy and cache it  */ 
             release_interface(&fd_table[fd]);
@@ -1158,8 +1204,11 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_get_device_id(mud
       }
    }
 
-   memcpy(buf, pd->id, *len > size ? size : *len); 
-   stat = HPMUD_R_OK;
+   if (*len)
+   {
+      memcpy(buf, pd->id, *len > size ? size : *len); 
+      stat = HPMUD_R_OK;
+   }
 
    pthread_mutex_unlock(&pd->mutex);
    return stat;
@@ -1190,7 +1239,7 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_get_device_status
       if (fd == FD_NA)
       {
          /* Device not in use. Claim interface, but release for other processes. */
-         if ((fd = claim_id_interface(libusb_device)) != FD_NA)
+         if ((fd = claim_id_interface(libusb_device)) != MAX_FD)
          {
             r = device_status(fd, status);
             release_interface(&fd_table[fd]);
@@ -1293,6 +1342,8 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_raw_channel_open(
    int fd = FD_7_1_2;
    enum HPMUD_RESULT stat = HPMUD_R_DEVICE_BUSY;
 
+   get_interface(libusb_device, fd, &fd_table[fd]);
+
    if (claim_interface(libusb_device, &fd_table[fd]))
       goto bugout;
 
@@ -1448,17 +1499,15 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_mlc_channel_open(
    /* Initialize MLC transport if this is the first MLC channel. */
    if (pd->channel_cnt==1)
    {
-      /* If 7/1/3 (MLC/1284.4) protocol is available use it. */
-      if (get_interface(libusb_device, FD_7_1_3, &fd_table[FD_7_1_3]) == 0)
+      if (get_interface(libusb_device, FD_7_1_3, &fd_table[FD_7_1_3]) == 0 && claim_interface(libusb_device, &fd_table[FD_7_1_3]) == 0)
          fd = FD_7_1_3;    /* mlc, dot4 */
-      else if (get_interface(libusb_device, FD_ff_ff_ff, &fd_table[FD_ff_ff_ff]) == 0)
+      else if (get_interface(libusb_device, FD_ff_ff_ff, &fd_table[FD_ff_ff_ff]) == 0 && claim_interface(libusb_device, &fd_table[FD_ff_ff_ff]) == 0)
          fd = FD_ff_ff_ff;   /* mlc, dot4 */
-      else if (get_interface(libusb_device, FD_ff_d4_0, &fd_table[FD_ff_d4_0]) == 0)
+      else if (get_interface(libusb_device, FD_ff_d4_0, &fd_table[FD_ff_d4_0]) == 0 && claim_interface(libusb_device, &fd_table[FD_ff_d4_0]) == 0)
          fd = FD_ff_d4_0;   /* mlc, dot4 */
-      else 
+      else if (get_interface(libusb_device, FD_7_1_2, &fd_table[FD_7_1_2]) == 0 && claim_interface(libusb_device, &fd_table[FD_7_1_2]) == 0)
          fd = FD_7_1_2;    /* raw, mlc, dot4 */
-
-      if (claim_interface(libusb_device, &fd_table[fd]))
+      else
       {
          stat = HPMUD_R_DEVICE_BUSY;
          goto bugout;
@@ -1670,22 +1719,20 @@ enum HPMUD_RESULT __attribute__ ((visibility ("hidden"))) musb_dot4_channel_open
    /* Initialize MLC transport if this is the first MLC channel. */
    if (pd->channel_cnt==1)
    {
-      /* If 7/1/3 (MLC/1284.4) protocol is available use it. */
-      if (get_interface(libusb_device, FD_7_1_3, &fd_table[FD_7_1_3]) == 0)
+      if (get_interface(libusb_device, FD_7_1_3, &fd_table[FD_7_1_3]) == 0 && claim_interface(libusb_device, &fd_table[FD_7_1_3]) == 0)
          fd = FD_7_1_3;    /* mlc, dot4 */
-      else if (get_interface(libusb_device, FD_ff_ff_ff, &fd_table[FD_ff_ff_ff]) == 0)
+      else if (get_interface(libusb_device, FD_ff_ff_ff, &fd_table[FD_ff_ff_ff]) == 0 && claim_interface(libusb_device, &fd_table[FD_ff_ff_ff]) == 0)
          fd = FD_ff_ff_ff;   /* mlc, dot4 */
-      else if (get_interface(libusb_device, FD_ff_d4_0, &fd_table[FD_ff_d4_0]) == 0)
+      else if (get_interface(libusb_device, FD_ff_d4_0, &fd_table[FD_ff_d4_0]) == 0 && claim_interface(libusb_device, &fd_table[FD_ff_d4_0]) == 0)
          fd = FD_ff_d4_0;   /* mlc, dot4 */
-      else 
+      else if (get_interface(libusb_device, FD_7_1_2, &fd_table[FD_7_1_2]) == 0 && claim_interface(libusb_device, &fd_table[FD_7_1_2]) == 0)
          fd = FD_7_1_2;    /* raw, mlc, dot4 */
-
-      if (claim_interface(libusb_device, &fd_table[fd]))
+      else
       {
          stat = HPMUD_R_DEVICE_BUSY;
          goto bugout;
       }
-
+         
       if (fd == FD_7_1_2)
       { 
          if (pd->io_mode == HPMUD_DOT4_BRIDGE_MODE)
@@ -1923,6 +1970,7 @@ int __attribute__ ((visibility ("hidden"))) musb_probe_devices(char *lst, int ls
    char rserial[128];
    char model[128];
    char serial[128];
+   char mfg[128];
    char sz[HPMUD_LINE_SIZE];
    int r, size=0;
 
@@ -1940,7 +1988,7 @@ int __attribute__ ((visibility ("hidden"))) musb_probe_devices(char *lst, int ls
             continue;
          }
 
-         model[0] = serial[0] = rmodel[0] = rserial[0] = sz[0] = 0;
+         model[0] = serial[0] = rmodel[0] = rserial[0] = sz[0] = mfg[0] = 0;
 
          if (dev->descriptor.idVendor == 0x3f0 && is_interface(dev, 7))
          {
@@ -1955,6 +2003,11 @@ int __attribute__ ((visibility ("hidden"))) musb_probe_devices(char *lst, int ls
             else
                generalize_serial(rserial, serial, sizeof(serial));
 
+            if ((r=get_string_descriptor(hd, dev->descriptor.iManufacturer, sz, sizeof(sz))) < 0)
+               BUG("invalid manufacturer string ret=%d\n", r);
+            else
+               generalize_serial(sz, mfg, sizeof(serial));
+
             if (!serial[0])
                strcpy(serial, "0"); /* no serial number, make it zero */
 
@@ -1975,11 +2028,11 @@ int __attribute__ ((visibility ("hidden"))) musb_probe_devices(char *lst, int ls
                 * This will allow us to do discovery and not disable other CUPS backend(s) who use /dev/usb/lpx instead of libusb.
                 */
                if (strncasecmp(rmodel, "hp ", 3) == 0)
-                  size += snprintf(lst+size, lst_size-size, "direct %s \"HP %s\" \"HP %s USB %s HPLIP\" \"MFG:HP;MDL:%s;CLS:PRINTER;DES:%s;SN:%s;\"\n", 
-                                                      sz, &rmodel[3], &rmodel[3], serial, rmodel, rmodel, rserial);
+                  size += snprintf(lst+size, lst_size-size, "direct %s \"HP %s\" \"HP %s USB %s HPLIP\" \"MFG:%s;MDL:%s;CLS:PRINTER;DES:%s;SN:%s;\"\n", 
+                                   sz, &rmodel[3], &rmodel[3], serial, mfg, rmodel, rmodel, rserial);
                else
-                  size += snprintf(lst+size, lst_size-size, "direct %s \"HP %s\" \"HP %s USB %s HPLIP\" \"MFG:HP;MDL:%s;CLS:PRINTER;DES:%s;SN:%s;\"\n", 
-                                                   sz, rmodel, rmodel, serial, rmodel, rmodel, rserial);
+                  size += snprintf(lst+size, lst_size-size, "direct %s \"HP %s\" \"HP %s USB %s HPLIP\" \"MFG:%s;MDL:%s;CLS:PRINTER;DES:%s;SN:%s;\"\n", 
+                                   sz, rmodel, rmodel, serial, mfg, rmodel, rmodel, rserial);
 
                *cnt+=1;
             }
@@ -2066,4 +2119,37 @@ bugout:
    return stat;
 }
 
+enum HPMUD_RESULT hpmud_make_usb_serial_uri(const char *sn, char *uri, int uri_size, int *bytes_read)
+{
+   struct usb_bus *bus;
+   struct usb_device *dev, *found_dev=NULL;
+   char model[128];
+   enum HPMUD_RESULT stat = HPMUD_R_INVALID_DEVICE_NODE;
+
+   DBG("[%d] hpmud_make_usb_serial_uri() sn=%s\n", getpid(), sn);
+
+   *bytes_read=0;
+
+   usb_init();
+   usb_find_busses();
+   usb_find_devices();
+
+   for (bus=usb_busses; bus && !found_dev; bus=bus->next)
+      for (dev=bus->devices; dev && !found_dev; dev=dev->next)
+        if (is_serial(dev, sn, model, sizeof(model)))
+            found_dev = dev;  /* found usb device that matches serial number */
+
+   if (found_dev == NULL)
+   {
+      BUG("invalid sn %s\n", sn);
+      goto bugout;
+   }
+ 
+   *bytes_read = snprintf(uri, uri_size, "hp:/usb/%s?serial=%s", model, sn); 
+   stat = HPMUD_R_OK;
+
+bugout:
+   return stat;
+}
+
 
