@@ -1,7 +1,7 @@
 /*****************************************************************************\
   context.h : Interface/Implimentation for the PrintContext class
 
-  Copyright (c) 1996 - 2006, Hewlett-Packard Co.
+  Copyright (c) 1996 - 2008, Hewlett-Packard Co.
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,22 @@
 #ifndef APDK_PRINTCONTEXT_H
 #define APDK_PRINTCONTEXT_H
 
+#define MEDIASIZE_PCL 0
+typedef struct
+{
+    int      pcl_id;
+    float    fPhysicalWidth;
+    float    fPhysicalHeight;
+    float    fPrintableWidth;
+    float    fPrintableHeight;
+    float    fPrintableStartX;
+    float    fPrintableStartY;
+} MediaAttributes;
+typedef struct
+{
+    MediaAttributes    media_attributes;
+} JobAttributes;
+
 #include "printerfactory.h"
 
 APDK_BEGIN_NAMESPACE
@@ -232,8 +248,24 @@ public:
 //    PEN_TYPE GetCompatiblePen(unsigned int num);    // get CompatiblePens of Printer
 
 #ifdef APDK_AUTODUPLEX
+    //! Set two-sided printing option.
+    /*!  This has no effect on printers that do not support an automatic duplexer mechanism.
+    Two sided-printing option can be one of
+        DUPLEXMODE_NONE   - one sided printing
+        DUPLEXMODE_TABLET - binding on short edge
+        DUPLEXMODE_BOOK   - binding on long edge
+	*****************************************************************************
+	*/   
     BOOL  SelectDuplexPrinting (DUPLEXMODE duplexmode);
+
+    //! Returns current setting for two-sided printing option
     DUPLEXMODE QueryDuplexMode ();
+
+    //! Does back page needs to be rotated?
+    /*! When automatic two-sided printing is enabled for long edge binging, application may have to
+    rotate the back page by 180 degrees.
+	*****************************************************************************
+	*/   
     BOOL  RotateImageForBackPage ();
 #endif
 
@@ -246,17 +278,38 @@ public:
 
     void    ResetIOMode (BOOL bDevID, BOOL bStatus);
 
+    //! Get number of copies to print
     int     GetCopyCount ()
     {
         return m_iCopyCount;
     }
+
+    //! Set number of copies to print.
+    /*! Copy count has no effect on printers that do not support this feature.
+    Typically, most LaserJet printers support multiple copies, whereas, Inkjets do not.
+	*****************************************************************************
+	*/   
     void    SetCopyCount (int iNumCopies)
     {
         m_iCopyCount = iNumCopies;
     }
-#ifdef APDK_LINUX
-    DRIVER_ERROR    SetPrinterHint (int iHint, int iValue);
-#endif
+    DRIVER_ERROR    SetPrinterHint (PRINTER_HINT eHint, int iValue);
+
+    //! Returns TRUE if borderless printing is enabled, FALSE otherwise
+    BOOL    IsBorderless () { return bDoFullBleed; }
+
+    //! SetMediaType
+    /*! Typically, media type is bound to a print mode and is set when a printmode is
+    selected. This is because the mediatype, printmode combination affects colormatching
+    and requires a different colormap data. Some printers do colormatching in firmware
+    and may support multiple mediatypes for a selected printmode.
+    Returns NO_ERROR if requested mediatype is supported for the selected printmode, otherwise
+    returns WARN_PRINTMODE_MISMATCH and the mediatype remains unchanged in the printmode.
+	*****************************************************************************
+	*/
+    DRIVER_ERROR    SetMediaType (MEDIATYPE eMediaType);
+    int             GetJobAttributes (int getWhat);
+    void            SetJobAttributes (JobAttributes *pJA);
 
 private:
 
@@ -312,6 +365,7 @@ private:
 */
 
     MEDIATYPE   m_mtReqMediaType;   // for use by Header - Malibu defect
+    MEDIATYPE   m_eMediaType;
 
 #ifdef APDK_AUTODUPLEX
     DUPLEXMODE  DuplexMode;
@@ -324,6 +378,8 @@ private:
 
     MediaSource m_MediaSource;
 
+    JobAttributes    *m_job_attributes;
+
 #ifdef APDK_CAPTURE
     void Capture_PrintContext(unsigned int InputPixelsPerRow, unsigned int OutputPixelsPerRow,
                               PAPER_SIZE ps,IO_MODE IOMode);
