@@ -45,8 +45,12 @@ APDK_BEGIN_NAMESPACE
 extern uint32_t ulMapDJ600_CCM_K[ 9 * 9 * 9 ];
 
 const char GrayscaleSeq[]= {ESC, '*', 'o', '5', 'W', 0x0B, 0x01, 0x00, 0x00, 0x02};
+
+const BYTE ExtraDryTime[] = "\033&b16WPML \004\000\006\001\004\001\004\001\006\010\001";
+
 extern BYTE EscAmplCopy(BYTE *dest, int num, char end);
 extern void AsciiHexToBinary(BYTE* dest, char* src, int count);
+extern MediaType MediaTypeToPcl (MEDIATYPE eMediaType);
 
 DJ9xxVIP::DJ9xxVIP
 (
@@ -92,6 +96,7 @@ DJ9xxVIP::DJ9xxVIP
     pMode[ModeCount++] = new DJ990BestMode ();         // Photo Best
     pMode[ModeCount++] = new DJ990PhotoNormalMode ();  // Photo Normal
 
+    m_cExtraDryTime = 0;
 }
 
 GrayModeDJ990::GrayModeDJ990
@@ -496,6 +501,19 @@ DRIVER_ERROR HeaderDJ990::Send()
     BOOL            bDeviceText;
 
     thePrintContext->GetPrintModeSettings (eQualityMode, eMediaType, eColorMode, bDeviceText);
+
+#ifdef APDK_EXTENDED_MEDIASIZE
+/*
+ *  A different media type may have been set via ppd that is different than
+ *  what is initialized in the PrintMode constructor, for example, eMediaType
+ *  set to 2, which maps to PCL mediatype of 3, rather than PCL mediatype 5
+ *  for 1200 dpi mode for Officejet Pro K5400. Here, set the media type based
+ *  on what was selected from ppd.
+ */
+
+    SetMediaType (MediaTypeToPcl (eMediaType));
+#endif
+
     if (eMediaType == MEDIA_CDDVD)
     {
         thePrintContext->SetMediaSource (sourceTrayCDDVD);
@@ -508,6 +526,14 @@ DRIVER_ERROR HeaderDJ990::Send()
     if (thePrintContext->QueryDuplexMode () != DUPLEXMODE_NONE)
     {
         err = thePrinter->Send ((const BYTE *) EnableDuplex, sizeof (EnableDuplex));
+        BYTE    cDryTime;
+        cDryTime = (BYTE) ((thePrinter->GetHint (EXTRA_DRYTIME_HINT)) & 0xFF);
+        if (cDryTime != 0)
+        {
+            err = thePrinter->Send (ExtraDryTime, sizeof (ExtraDryTime) - 1);
+            err = thePrinter->Send ((const BYTE *) &cDryTime, 1);
+            ERRCHECK;
+        }
     }
 #endif
 
@@ -626,6 +652,9 @@ DRIVER_ERROR HeaderDJ990::Send()
     err = thePrinter->Send ((const BYTE *) "\033&l-2H", 6);   // Moved from Modes(), des 3/11/03
     ERRCHECK;
 
+//  Send speed mech command
+    thePrinter->SetHint (SPEED_MECH_HINT, 0);
+
     // no need for compression command, it's in the CRD
 
     err = thePrinter->Send((const BYTE*)grafStart, sizeof(grafStart) );
@@ -832,6 +861,10 @@ DRIVER_ERROR DJ9xxVIP::ParsePenInfo(PEN_TYPE& ePen, BOOL QueryPrinter)
     {
         num_pens = 10 + (str[0] - 'A');
     }
+    else if ((str[0] >= 'a') && (str[0] <= 'f'))
+    {
+        num_pens = 10 + (str[0] - 'a');
+    }
     else
     {
         return BAD_DEVICE_ID;
